{: This is the core of the wrappers used for supporting Borland Delphi.
  This unit is where the static parent classes are all defined.  The other
  unit file of importance for the stubcode is generated dynamically and
  will have the name &lt;prefix&gt;.pas }
unit CHM;

interface

uses
  Windows, SysUtils, Classes, Dialogs;

// This file ships with Chameleon.  All the generated classes inherit
// off base classes defined in this file.  The idea is that advanced users
// of Chameleon may wish to alter the behavior of the message classes by
// modifying these base classes.
//
// If this is done, care should be taken to ensure that the changes are not
// lost when a new copy of Chameleon is installed.  We suggest storing
// CHM.pas under what ever source code control system that you use in your
// organization.
//
// These classes form a thin wrapper over a set of basic C API functions
// implemented in the CHM_LIB3.dll.
{$IFDEF VER150}
    {$WARN UNSAFE_CODE OFF}
{$ENDIF}

{$IFNDEF HIDE}

type TCHMresultHandle = Pointer;
type TCHMtableHandle = Pointer;
type TCHMengineHandle = Pointer;
type TCHMdateTimeHandle = Pointer;

type TCardinalPtr = ^Cardinal;
type TIntegerPtr = ^Integer;
type TBoolPtr = ^Bool;
type TShortIntPtr = ^ShortInt;
type TDoublePtr = ^Double;
type TDatePtr = ^TDateTime;
type TPAnsiCharPtr = ^PAnsiChar;
type TLongIntPtr = ^LongInt;
type TCHMengineHandlePtr = ^TCHMengineHandle;
type TCHMtableHandlePtr = ^TCHMtableHandle;
type TCHMdateTimeHandlePtr = ^TCHMdateTimeHandle;

const CHM_OK = 0;

const  CHM_DEF_peRequiredFieldNotPresent     = 0;
const  CHM_DEF_peFieldMaxRepeatsExceeded     = 1;
const  CHM_DEF_peUnexpectedExtraFields       = 2;
const  CHM_DEF_peFieldMaxLengthExceeded      = 3;
const  CHM_DEF_peUnexpectedSegment           = 4;
const  CHM_DEF_peRequiredSegmentMissing      = 5;
const  CHM_DEF_peRequiredSegmentGroupMissing = 6;
const  CHM_DEF_peUnexpectedTrailingSegments  = 7;
const  CHM_DEF_peBadEscapeSequence           = 8;
const  CHM_DEF_peNoSegmentIdentifier         = 9;
const  CHM_DEF_peBadIntegerFormat            = $0A;
const  CHM_DEF_peBadDoubleFormat             = $0B;
const  CHM_DEF_peBadDateTimeFormat           = $0C;

// Exceptions - used as codes in the CHMerror interface
const  CHM_DEF_ecPreCondition        =$80000100;
const  CHM_DEF_ecPostCondition       =$80000101;
const  CHM_DEF_ecFileExists          =$80000200;
const  CHM_DEF_ecFileError           =$80000201;
const  CHM_DEF_ecBoundaryError       =$80000300;
const  CHM_DEF_ecTableContainsData   =$80000400;
const  CHM_DEF_ecSystemError         =$80000500;
const  CHM_DEF_ecInvalidDateTime     =$80000600;
const  CHM_DEF_ecInvalidRepeatChar   =$80000700;
const  CHM_DEF_ecIllegalOperation    =$80000800;
const  CHM_DEF_ecWrongDataType       =$80000801;
const  CHM_DEF_ecUnknownDataType     =$80000900;
const  CHM_DEF_ecSerializationError  =$80001000;
const  CHM_DEF_ecMessageTooShort     =$80001100;
const  CHM_DEF_ecNoHeaderSegment     =$80001200;
const  CHM_DEF_ecNoEncodingCharField =$80001201;
const  CHM_DEF_ecNonExistentSubTable =$80001300;
const  CHM_DEF_ecNoMessagesLoaded    =$80001400;

// Parse Results - set of exception codes from the parsing routine
const CHM_DEF_prUnknownMessage      =$80002001;
const CHM_DEF_prUnknownSegment      =$80002002;
const CHM_DEF_prBadSegmentGrammar   =$80002003;
const CHM_DEF_prBadMessageGrammar   =$80002004;
const CHM_DEF_prNoSegmentIdentifier =$80002005;
{$ENDIF}

{: String type. }
const __CHM_STRING_TYPE    = 0;
{: Integer type. }
const __CHM_INTEGER_TYPE   = 1;
{: Double type. }
const __CHM_DOUBLE_TYPE    = 2;
{$IFNDEF HIDE}
const __CHM_COMPOSITE_TYPE = 3;
{$ENDIF}
{: Date Time type. }
const __CHM_DATE_TIME_TYPE = 4;

{: Table State for cell data. }
const CHM_TABLE_STATE_VALID = 0;
const CHM_TABLE_STATE_PRESENT_BUT_NULL =  1;
const CHM_TABLE_STATE_EMPTY = 2;
const CHM_TABLE_STATE_INVALID =  3;
			    
{$IFDEF VER100}
type TErrorCode = Longint;
{$ELSE}
{: This type gives the error number during
parsing errors.
Under Delphi 3.0 this is a <InlineCode>Longint</InlineCode> rather than a
<InlineCode>Longword</InlineCode>.  }
type TErrorCode = Longword;
{$ENDIF}

{: Get the registration level of Chameleon.
@returns 0 if Chameleon is not registered <BR>1 if Chameleon is registered with a trial license
<BR>2 if Chameleon is registered with a trial extension license <BR>3 if Chameleon is registered with a full license}
function ChameleonRegistrationLevel : LongInt;

{$IFNDEF HIDE}
procedure CHMcheckResult(ReturnVal : TCHMresultHandle);
{$ENDIF}

{: This class handles Date Time objects used in tables for Chameleon. }
type TCHMdateTime = class(TObject)
     private
{$IFNDEF HIDE}
        FHandle : TCHMdateTimeHandle;
        function GetDateTime: TDateTime;
        function GetSecondFraction: Double;
        function GetZone: SmallInt;
        procedure SetDateTime(const Value: TDateTime);
        procedure SetSecondFraction(const Value: Double);
        procedure SetZone(const Value: SmallInt);
        function  GetIsNull : Boolean;
{$ENDIF}
     public
        {: Constructor.  Should not need to be overridden by the user.}
        constructor Create(Handle : TCHMdateTimeHandle);
        {: Destructor.}
        destructor Destroy; override;
        {: The handle to the underlying Chameleon object.  This is used in the internal
        implementation of Chameleon and should not be invoked directly. }
        property Handle : TCHMdateTimeHandle read FHandle;
        {: The fractional value of the seconds of the Date Time value.}
        property SecondFraction : Double read GetSecondFraction write SetSecondFraction;
        {: The floating point quantity that measures the number of days from midnight (12AM) 30 December 1899,
        the same as <InlineCode>TDateTime</InlineCode>. For example, 6AM 1 January 1900 is represented as 2.25.
        Hours and minutes are represented as fractional days, i.e., 0.25 * 24 hours = 6AM.}
        property DateTime : TDateTime read GetDateTime write SetDateTime;
        {: The time zone of the Date Time value.  For example, a TCHMdateTime with a time in the EST would
        have a Zone property of -5.}
        property Zone : SmallInt read GetZone write SetZone;
        {: Check whether the TCHMdateTime is null.}
        property IsNull : Boolean read GetIsNull;
     end;
     TCHMcustomEngine=class;
     {: Invoked when a log message is being written.  The log message
      is contained in the string <InlineCode>Line</InlineCode>. }
     TCHMlogEvent  = procedure(Sender: TObject; Line : AnsiString) of Object;
   {: Invoked just before a message is parsed.  This is
      useful for implementing logging functionality.  The message
      to be parsed is contained in the string <InlineCode>MessageData</InlineCode>. }
   TCHMmessageEvent = procedure(Sender: TObject; MessageData : AnsiString) of Object;
   {: Invoked when a parsing error occurs.  The user should implement
     proper logging behavior when this event is invoked.}
   TCHMparseErrorEvent = procedure(Sender: TObject;
                                  MessageData : AnsiString;
                                  TransportIndex : Integer;
                                  Description : AnsiString;
                                  Code : TErrorCode) of Object;
   {: This event is used to implement <See Method=TCHMdynamicEngine.ParseMessage>. }
   TCHMparseMessageEvent = procedure(Sender : TObject;
                                     MessageData : AnsiString;
                                     TransportIndex : Integer) of Object;
   {: This class represents an instance of a Chameleon engine.  Before the Engine can be
   used the programmer must load a visual message definition (VMD) file using <See Method=Load>.
   This class is the base class from which custom Engine classes inherit.}
   TCHMcustomEngine = class(TComponent)
  private
{$IFNDEF HIDE}
     FMappingFile : AnsiString;
     FEngineHandle : TCHMengineHandle;
     FOnLog: TCHMlogEvent;
     FStrictGrammarChecking : Boolean;
     FOnMessage: TCHMmessageEvent;
 //    FOnMessageSent: TCHMmessageEvent;
     FOnParseError: TCHMparseErrorEvent;
     procedure SetRejectBadSegmentGrammarFlag(const Value: Boolean);
    procedure SetOnLog(const Value: TCHMlogEvent);
    procedure SetOnMessage(const Value: TCHMmessageEvent);
    procedure SetOnParseError(const Value: TCHMparseErrorEvent);
    procedure SetMappingFile(const Value : AnsiString);
    function GetCountOfCOnfig : Integer;
    function GetIncomingConfigIndex : Integer;
    function GetOutgoingConfigIndex : Integer;
    function GetCurrentConfigIndex : Integer;
    function GetMajorVersion : Integer;
    function GetMinorVersion : Integer;
    function GetBuildNumber : Integer;
    procedure SetCurrentConfigIndex( Index : Integer );
{$ENDIF}
  public
     {: Invoked when an unidentified message is parsed.
       By default it invokes the <See Event=OnParseError> event, but it could be
       altered to have different behavior.}
     procedure HandleUnexpectedMessage(MessageData : AnsiString;
                     TransportIndex, MessageIndex : Integer);
     {: Invoked when an exception is raised.  It will normally invoke the
        <See Event=OnParseError> event.  If this event is not assigned then this method
        will write the exception message to the log. }
     procedure HandleException(MessageData : AnsiString;
                     TransportIndex : Integer;
                     ExceptionHandle : TCHMresultHandle);
     {: Constructor. }
     constructor Create(Owner : TComponent); override;
     {: Destructor. }
     destructor Destroy; override;
     {: Get the name of the message at index <InlineCode>MessageIndex</InlineCode>.}
     function MessageName(MessageIndex : Cardinal) : AnsiString;
     {: Parse flatwire messages into a table if stubcode is being used.
     This procedure must be overridden by child classes to do the actual parsing. <BR><BR>It can also be
     overridden for logging purposes, or for any other application where the user would need to access the
     flatwire format of the message. It also invokes the <See Event=OnMessage> event if this event has been assigned.}
     procedure ParseMessage(MessageData : AnsiString; TransportIndex : Integer); virtual;
     {: Transform a message from one format to another.
     This function invokes Chameleon's translation logic to take a message of the <InlineCode>InConfigIndex</InlineCode>
     flavor and translate it to the <InlineCode>OutConfigIndex</InlineCode> flavor.  The function returns the transformed
     data. <BR><BR>See <A HRef="http://www.interfaceware.com/manual/transform_message.html">http://www.interfaceware.com/manual/transform_message.html</A>
     for more information.}
     function TransformMessage(InMessage : AnsiString;
                               InConfigIndex, OutConfigIndex : Cardinal) : AnsiString;
     {: Transform an HL7 message into XML.  This translation is dependent on what is set in the XML tab of the Options
     window in the Chameleon GUI for your VMD.  <BR><BR>See <A HRef="http://www.interfaceware.com/manual/xml.html">http://www.interfaceware.com/manual/xml.html</A>
     for more information.}
     function TranslateHL7toXML(InMessage : AnsiString) : AnsiString;
     {: Transform an XML message into HL7.  This translation depends on what is set in the XML tab of the Options
     window in the Chameleon GUI for your VMD.  <BR><BR>See <A HRef="http://www.interfaceware.com/manual/xml.html">http://www.interfaceware.com/manual/xml.html</A>
     for more information.}
     function TranslateXMLtoHL7(InXML : AnsiString) : AnsiString;
     {: Write a message to the log.  Usage of the python <InlineCode>log()</InlineCode> command in the VMD file will be output to the log.
     The actual destination will depend on how the user has implemented the <See Event=OnLog> event.}
     procedure WriteToLog(Line : AnsiString);
     {: Load the visual message definition (VMD) file <InlineCode>FileName</InlineCode>.  This must be done before the
     Engine can be used.  <BR><BR>Applications should load the VMD file as part of initialization since loading may take a few seconds.
     Loading the VMD file every time a message is encountered will result in very slow performance.}
     procedure Load(FileName : AnsiString);
     {: Get the index of the message selected to be the ACKnowledgment message.  This message is set in the Options window of
     Chameleon in the Special Messages tab.  See <A HRef="http://www.interfaceware.com/manual/autoack.html">http://www.interfaceware.com/manual/autoack.html</A>
     for more information. }
     function AckMessageIndex : Cardinal;
     {: The handle to the underlying Chameleon engine object.  This is used in the internal implementation of Chameleon and should
     not be used directly.}
     property EngineHandle : TCHMengineHandle read FEngineHandle;
     {: Generate an ACKnowledgment message in an optimized manner.  This function strips off just the MSH segment of
     an HL7 message, parses this segment, stores parameters in python variables, then generates an ACK message.  The
     technique used is described as auto ACKnowledgment in the manual:
     <A HRef="http://www.interfaceware.com/manual/autoack.html">http://www.interfaceware.com/manual/autoack.html</A>.  <BR><BR>This function
     will have unpredictable results if the VMD file is not set up correctly for auto acknowledgment.}
     function GenerateAck(OriginalMessage : AnsiString) : AnsiString;
     {: Get the name of the configuration at the specified index. }
     function GetConfigName( Index : Integer ) : AnsiString;
     {: The number of configurations in the message definition (VMD) file. }
     property CountOfConfig : Integer read GetCountOfConfig;
     {: The index of the incoming configuration used for graphical transformation.  The index is that of the input configuration
     last set by the user in the Transformation window in the VMD file unless it is set to another value. <Strong>This is not used
     for scripted transformation</Strong>.}
     property IncomingConfigIndex : Integer read GetIncomingConfigIndex;
     {: The index of outgoing configuration used for graphical transformation.  The index is that of the output configuration
     last set by the user in the Transformation window in the VMD file unless it is set to another value. <Strong>This is
     not used for scripted transformation</Strong>. }
     property OutgoingConfigIndex : Integer read GetOutgoingConfigIndex;
     {: The index of the configuration currently selected. }
     property CurrentConfigIndex : Integer read GetCurrentConfigIndex write SetCurrentConfigIndex;
     {: The major version number of Chameleon.}
     property MajorVersion : Integer read GetMajorVersion;
     {: The minor version number of Chameleon. }
     property MinorVersion : Integer read GetMinorVersion;
     {: The build number of Chameleon. }
     property BuildNumber : Integer read GetBuildNumber;
  published

     {: The visual message definition (VMD) file.  This is another way of assigning the message definition file. }
     property MappingFile : AnsiString read FMappingFile write SetMappingFile;
     {: The option to reject bad segment grammar in a parsed message.  The default value is <InlineCode>false</InlineCode>.
     When this flag is set the parser will check for the presence of compulsory fields in each segment and enforce other
     rules such as maximum field length.}
     property StrictGrammarChecking : Boolean read FStrictGrammarChecking write SetRejectBadSegmentGrammarFlag default false;
     {: Fired every time a message is received.  This is a good place
       to put in logging code for your application.}
     property OnMessage: TCHMmessageEvent read FOnMessage write SetOnMessage;
     {: Fired when a parsing error occurs. }
     property OnParseError : TCHMparseErrorEvent read FOnParseError write SetOnParseError;
     {: Fired every time the <See Method=WriteToLog> method is called.  It should be overridden
     with code to write the log to an appropriate destination. }
     property OnLog : TCHMlogEvent read FOnLog write SetOnLog;
  end;
  TCHMcustomTable = class;
  {: This class handles Table rows.  This class is used as the parent for each custom generated table row class.}
  TCHMcustomTableRow = class(TObject)
     private
{$IFNDEF HIDE}
        FRowIndex : Integer;
        FTable : TCHMcustomTable;
        function GetSubTable(SubTableIndex : Integer): TCHMcustomTable;
     protected
        FDateTime : TCHMdateTime;
{$ENDIF}
     public
        {: Constructor.}
        constructor Create(Table : TCHMcustomTable; RowIndex : Integer);
        {: Destructor.}
        destructor Destroy ; override;
        {: Get the index of the Row.}
        function RowIndex : Integer;
        {: Get the number of sub-tables in this Row.}
        function CountOfSubTable : Integer;
        {: Get the handle to the underlying Table object. This is used in the internal
        implementation of Chameleon and should not be invoked directly.}
        function TableHandle : TCHMtableHandle;
        {: The sub-table of this Row. }
        property SubTable[SubTableIndex : Integer] : TCHMcustomTable read GetSubTable;
     end;
     {: This class handles Table objects.  This class gives the interface to the basic data
     container in the Chameleon framework: a nested in-memory table.
     All the generated table objects and messages inherit from this class.}
   TCHMcustomTable = class(TObject)
   private
{$IFNDEF HIDE}
      FTableHandle : TCHMtableHandle;
      FTableRow : TCHMcustomTableRow;
      FCountOfSubTable : Integer;
      FTableMatrix : Pointer;
      FDateTime : TCHMdateTime;
      FTableMatrixRowCount : Integer;
      procedure InitSubTables(RowIndexRequired : Integer);
      procedure DeleteSubTables;
{$ENDIF}
   protected
      {: Get the Row object at <InlineCode>RowIndex</InlineCode>. }
      function GetRow(RowIndex : Integer): TCHMcustomTableRow;
      {: Get the sub-table at <InlineCode>SubTableIndex</InlineCode> in the row
      at <InlineCode>RowIndex</InlineCode>. }
      function GetSubTable(RowIndex, SubTableIndex : Integer): TCHMcustomTable;
   public
      {: Constructor. }
      constructor Create(Handle : TCHMtableHandle);
      {: Destructor. }
      destructor Destroy; override;
      {: Get the number of rows in the Table.  This count should be checked before
      accessing any Row in the Table since a Table can be empty.}
      function CountOfRow : Integer;
      {: Get the number of columns in the Table.  Usage of this function with
      <See Method=ColumnName> offers an easy way to iterate through the columns
      in the Table.}
      function CountOfColumn : Integer;
      {: Get the data type of the column at <InlineCode>ColumnIndex</InlineCode> as an integer.
      This integer can be compared with the global constants <See Const=__CHM_STRING_TYPE>,  <See Const=__CHM_INTEGER_TYPE>,
      <See Const=__CHM_DOUBLE_TYPE>, <See Const=__CHM_DATE_TIME_TYPE> to determine the data type.
       <BR><BR>See <A HRef="http://www.interfaceware.com/manual/table_properties.html">http://www.interfaceware.com/manual/table_properties.html</A>
       for more information on table data types.}
      function ColumnType(ColumnIndex : Integer) : LongInt;
      {: Get the name of the column at <InlineCode>ColumnIndex</InlineCode>. Usage of this function with
      <See Method=CountOfColumn> offers an easy way to iterate through the columns in the Table.}
      function ColumnName(ColumnIndex : Integer) : AnsiString;
      {: Get the string value at <InlineCode>RowIndex</InlineCode> and <InlineCode>ColumnIndex</InlineCode>.
      See <A HRef="http://www.interfaceware.com/manual/table_properties.html">http://www.interfaceware.com/manual/table_properties.html</A>
      for more information on table data types. }
      function GetStringValue(RowIndex, ColumnIndex : Integer) : AnsiString;
      {: Get the integer value at <InlineCode>RowIndex</InlineCode> and <InlineCode>ColumnIndex</InlineCode>.
      See <A HRef="http://www.interfaceware.com/manual/table_properties.html">http://www.interfaceware.com/manual/table_properties.html</A>
      for more information on table data types. }
      function GetIntegerValue(RowIndex, ColumnIndex : Integer) : Integer;
      {: Get the double value at <InlineCode>RowIndex</InlineCode> and <InlineCode>ColumnIndex</InlineCode>.
      See <A HRef="http://www.interfaceware.com/manual/table_properties.html">http://www.interfaceware.com/manual/table_properties.html</A>
      for more information on table data types. }
      function GetDoubleValue(RowIndex, ColumnIndex : Integer) : double;
      {: Get the Date Time value at <InlineCode>RowIndex</InlineCode> and <InlineCode>ColumnIndex</InlineCode>.
      See <A HRef="http://www.interfaceware.com/manual/table_properties.html">http://www.interfaceware.com/manual/table_properties.html</A>
      for more information on table data types. }
      function GetDateTimeValue(RowIndex, ColumnIndex : Integer) : TCHMdateTime;
      {: Check if the value at <InlineCode>RowIndex</InlineCode> and <InlineCode>ColumnIndex</InlineCode> is null.
      See <A HRef="http://www.interfaceware.com/manual/is_null.html">http://www.interfaceware.com/manual/is_null.html</A> for
      more information. }
      function IsNull(RowIndex, ColumnIndex : Integer) : boolean;
      {: Test the value in the cell at row <InlineCode>RowIndex</InlineCode> and column <InlineCode>ColumnIndex</InlineCode> to see if it is null, zero, empty or invalid.
      See <A HRef="http://interfaceware.com/manual/is_null.html">http://interfaceware.com/manual/is_null.html</A> for details. }
      function GetState(RowIndex, ColumnIndex :  Integer) : Integer;
      {: Output the Table's contents to a string. This is useful for diagnostics.}
      function Dump : AnsiString;
      {: Add a row to the Table.  This function returns the index of the added row. }
      function AddRow : Integer;
      {: Get the number of sub-tables. }
      function CountOfSubTable : Integer;
      {: Get the handle of the underlying Table object. This is used in the internal implementation of Chameleon
      and should not be invoked directly.}
      function TableHandle : TCHMtableHandle;
      {: Get the name of the Table.}
      function TableName : AnsiString;
      {: The Row object for the row at <InlineCode>RowIndex</InlineCode>. }
      property Row[RowIndex : Integer] : TCHMcustomTableRow read GetRow;
      {: The sub-table at <InlineCode>SubTableIndex</InlineCode> in the row at <InlineCode>RowIndex</InlineCode>. }
      property SubTable[RowIndex, SubTableIndex : Integer] : TCHMcustomTable read GetSubTable;
   end;
   {: This class handles Message objects.  Each message object inherits from this class.  Each message is
      in fact a specialized instance of a table object. }
   TCHMcustomMessage = class(TCHMcustomTable)
   private
{$IFNDEF HIDE}
      FEngineHandle : TCHMengineHandle;
      FMessageIndex : Integer;
      function AddRow : Integer;
    procedure SetEngineHandle(const Value: TCHMengineHandle);  // not allowed since it is done automatically in the constructor
{$ENDIF}
   public
      {: This function is invoked to generate the actual message. }
      function GenerateFlatWire : AnsiString;
      {: Constructor. Used to construct an empty Message at index <InlineCode>MessageIndex</InlineCode>. }
      constructor CreateFromMessageIndex(Engine : TCHMcustomEngine;
                         iMessageIndex : Integer);
      {: Constructor. Used to construct a Message from a Table already populated with data.  This is used during
     parsing operations.}
      constructor Create(iEngineHandle : TCHMengineHandle;
                         iTableHandle  : TCHMtableHandle;
                         iMessageIndex : Integer);
      {: The handle to the underlying Engine object. This is used in the internal implementation of
      Chameleon and should not be used directly.}
      property EngineHandle : TCHMengineHandle read FEngineHandle write SetEngineHandle;

   end;
   {: This class implements a dynamic version of the Engine that allows the message tables to be explored
   dynamically rather than relying on stubcode.  This is required because <See Class=TCHMcustomEngine> has abstract methods.
   See <A HRef="http://www.interfaceware.com/manual/delphi_dynamic_interface.html">http://www.interfaceware.com/manual/delphi_dynamic_interface.html</A>
   for more information.}
   TCHMdynamicEngine = class (TCHMcustomEngine)
{$IFNDEF HIDE}
      protected
         FOnParseMessage : TCHMparseMessageEvent;
         procedure SetOnParseMessage(Event : TCHMparseMessageEvent);
{$ENDIF}
      public
         {: This method will trigger the <See Event=OnParseMessage> event.  This allows
       the user to specify how messages that are received from the transport
       framework should be handled. }
         procedure ParseMessage(MessageData : AnsiString; TransportIndex : Integer); override;
     {: Dynamically parse a message.  Allows the user to parse a message without invoking virtual methods
     in stubcode.  This is useful for auto-acknowledgment.  <BR><BR>An important thing to note is that you must first create
     a new <See Class=TCHMdynamicMessage> object before calling this function.
     <BR><BR>If successful the MessageTable object will be populated with data from the message.
     The function returns the index number of the message.
     <BR><BR>See <A HRef="http://www.interfaceware.com/manual/delphi_dynamic_interface.html">http://www.interfaceware.com/manual/delphi_dynamic_interface.html</A>
     for more information. }
         function ParseMessageDynamically(MessageData: AnsiString; MessageTable : TCHMcustomMessage) : Integer;
      published
         {: Triggered every time a message is parsed. }
         property OnParseMessage : TCHMparseMessageEvent read FOnParseMessage write SetOnParseMessage;
      end;
   {: This class handles Message objects when Chameleon is used without stubcode.  See <See Class=TCHMdynamicEngine>. }
   TCHMdynamicMessage = class (TCHMcustomMessage)
      public
      {: Constructor. Create an empty message table.}
      constructor Create(Engine : TCHMcustomEngine);
      {: Constructor. Create a empty message of the type at <InlineCode>MessageIndex</InlineCode>. }
      constructor CreateFromMessageIndex(Engine : TCHMcustomEngine; MessageIndex : Integer);
      end;

{$IFNDEF HIDE}
// These are needed for supporting log callbacks from the engine.
type SVCengineLogProc = procedure(LogHandle : Pointer; Line : PAnsiChar); stdcall;
type SVClogProc = procedure(Line : PAnsiChar); stdcall;
{$ENDIF}

// Date Time interface

{$IFNDEF HIDE}

function CHMdateTimeCreate
(
   HandlePtr : TCHMdateTimeHandlePtr
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name '_CHMdateTimeCreate@4';

function CHMdateTimeAddRef
(
   Handle : TCHMdateTimeHandle
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMdateTimeAddRef@4';


function CHMdateTimeRelease
(
   Handle : TCHMdateTimeHandle
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMdateTimeRelease@4';


function CHMdateTimeGetDATE
(
   Handle : TCHMdateTimeHandle;
   pValue : TDatePtr
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMdateTimeGetDATE@8';


function CHMdateTimeGetSecondFraction
(
   Handle : TCHMdateTimeHandle;
   pValue : TDoublePtr
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMdateTimeGetSecondFraction@8';


function CHMdateTimeGetZone
(
   Handle : TCHMdateTimeHandle;
   pValue : TIntegerPtr
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMdateTimeGetZone@8';

function CHMdateTimeGetIsNull
(
   Handle : TCHMdateTimeHandle;
   pValue : TShortIntPtr
) : TCHMresultHandle; stdcall;
external 'CHM_LIB3.dll' name '_CHMdateTimeGetIsNull@8';


function CHMdateTimeSetDATE
(
   Handle : TCHMdateTimeHandle;
   NewValue : TDateTime
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMdateTimeSetDATE@12';


function CHMdateTimeSetSecondFraction
(
   Handle : TCHMdateTimeHandle;
   NewValue : Double
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMdateTimeSetSecondFraction@12';


function CHMdateTimeSetZone
(
   Handle : TCHMdateTimeHandle;
   NewValue : SmallInt
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMdateTimeSetZone@8';

// Table interface

function CHMtableCreate
(
   HandlePtr : TCHMtableHandlePtr
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMtableCreate@4';


function CHMtableAddRef
(
   Handle : TCHMtableHandle
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMtableAddRef@4';


function CHMtableRelease
(
   Handle : TCHMtableHandle
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMtableRelease@4';


function CHMtableName
(
   Handle : TCHMtableHandle;
   pName  : TPAnsiCharPtr
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMtableName@8';


function CHMtableSubTable
(
   Handle     : TCHMtableHandle;
   RowIndex   : Integer;
   TableIndex : Integer;
   pSubTable  : TCHMtableHandlePtr
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMtableSubTable@16';

function CHMtableCountOfColumn
(
   Handle : TCHMtableHandle;
   Count  : TLongIntPtr
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMtableCountOfColumn@8';

function CHMtableCountOfRow
(
   Handle : TCHMtableHandle;
   pCount : TLongIntPtr
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMtableCountOfRow@8';


function CHMtableCountOfSubTable
(
   Handle : TCHMtableHandle;
   Count  : TLongIntPtr
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMtableCountOfSubTable@8';


function CHMtableColumnName
(
   Handle : TCHMtableHandle;
   ColumnIndex : LongInt;
   ppName       : TPAnsiCharPtr
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMtableColumnName@12';

function CHMtableColumnType
(
    Handle : TCHMtableHandle;
    ColumnIndex : LongInt;
    pDataType : TLongIntPtr  // do not know if enumeration would work here
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMtableColumnType@12';


function CHMtableIsNull
(
   Handle : TCHMtableHandle;
   ColumnIndex, RowIndex: LongInt;
   pIsNull : TBoolPtr
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMtableIsNull@16';

function CHMtableState
(
   Handle : TCHMtableHandle;
   ColumnIndex, RowIndex: LongInt;
   pOut : TLongIntPtr
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMtableState@16';

function CHMtableGetString
(
   Handle : TCHMtableHandle;
   ColumnIndex, RowIndex: LongInt;
   ppString : TPAnsiCharPtr
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMtableGetString@16';

function CHMtableGetDouble
(
   Handle : TCHMtableHandle;
   ColumnIndex, RowIndex : LongInt;
   pValue : TDoublePtr
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMtableGetDouble@16';


function CHMtableGetInteger
(
   Handle : TCHMtableHandle;
   ColumnIndex, RowIndex : LongInt;
   pInteger : TIntegerPtr
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMtableGetInteger@16';


function CHMtableGetDateTime
(
   Handle : TCHMtableHandle;
   ColumnIndex, RowIndex : LongInt;
   pDateTime : TCHMdateTimeHandlePtr
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name '_CHMtableGetDateTime@16';


function CHMtableSetString
(
   Handle : TCHMtableHandle;
   ColumnIndex, RowIndex : LongInt;
   Value : PAnsiChar
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name '_CHMtableSetString@16';


function CHMtableSetDouble
(
   Handle : TCHMtableHandle;
   ColumnIndex, RowIndex : LongInt;
   Value : Double
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name '_CHMtableSetDouble@20';


function CHMtableSetInteger
(
   Handle : TCHMtableHandle;
   ColumnIndex, RowIndex : LongInt;
   Value : LongInt
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMtableSetInteger@16';


function CHMtableSetDateTime
(
   Handle : TCHMtableHandle;
   ColumnIndex, RowIndex : LongInt;
   Value : TCHMdateTimeHandle
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMtableSetDateTime@16';


function CHMtableDump
(
   Handle : TCHMtableHandle;
   ppContent : TPAnsiCharPtr
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMtableDump@8';

function CHMtableFreeBuffer
(
   ppContent : TPAnsiCharPtr
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMtableFreeBuffer@4';

function CHMtableAddRow
(
   Handle : TCHMtableHandle
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
'_CHMtableAddRow@4';

{ TCHMcustomEngine }

function CHMengineCreate(HandlePtr : TCHMengineHandlePtr)
 : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
 '_CHMengineCreate@4';

function CHMengineAddRef(Handle : TCHMengineHandle)
 : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
 '_CHMengineAddRef@4';

function CHMengineRelease(Handle : TCHMengineHandle)
 : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
 '_CHMengineRelease@4';

function CHMengineLoad (Handle : TCHMengineHandle; pFileName : PAnsiChar)
 : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
 '_CHMengineLoad@8';

function CHMengineGetAckMessageIndex(
  Handle : TCHMengineHandle;
  pMessageIndex : TLongIntPtr) : TCHMresultHandle; stdcall;
  external 'CHM_LIB3.dll' name '_CHMengineGetAckMessageIndex@8';

function CHMengineGenerateAckFromMessage(
  Handle : TCHMengineHandle;
  pOriginalMessage : PAnsiChar;
  ppAckMessage : TPAnsiCharPtr) : TCHMresultHandle; stdcall;
  external 'CHM_LIB3.dll' name '_CHMengineGenerateAckFromMessage@12';

function CHMengineParseMessage(Handle          : TCHMengineHandle;
                               FlatWire        : PAnsiChar;
                               MessageIndexPtr : TLongIntPtr;
                               ResultTable     : TCHMtableHandle)
 : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name '_CHMengineParseMessage@16';

function CHMengineTransformMessage(Handle         : TCHMengineHandle;
                                   InMessage      : PAnsiChar;
                                   InConfigIndex  : Cardinal;
                                   OutConfigIndex : Cardinal;
                                   OutMessage     : TPAnsiCharPtr;
                                   MessageIndex   : TCardinalPtr;
                                   IsDebugging    : Cardinal
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name '_CHMengineTransformMessage@28';

function CHMengineTranslateHL7toXML(Handle : TCHMengineHandle;
                                    pMessage : PAnsiChar;
                                    ppResult : TPAnsiCharPtr)
   : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name '_CHMengineTranslateMessageToHl7StandardXml@12';

function CHMengineTranslateXMLtoHL7(Handle : TCHMengineHandle;
                                    pXML : PAnsiChar;
                                    ppMessage : TPAnsiCharPtr)
   : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name '_CHMengineTranslateHl7StandardXmlToMessage@12';

// The MessageTable must have been setup by createEmptyTable

function CHMengineGenerateMessage(Handle : TCHMengineHandle; ppFlatWire : TPAnsiCharPtr;
   MessageIndex : LongInt; MessageTable : TCHMtableHandle)
 : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
 '_CHMengineGenerateMessage@16';

// This function is used to generate an empty table for a message.

function CHMengineCreateEmptyMessageTable(Handle : TCHMengineHandle;
 MessageIndex : LongInt; EmptyTable : TCHMtableHandle)
 : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
 '_CHMengineCreateEmptyMessageTable@12';

function CHMengineSetLogCallbackPerEngine(Handle : TCHMengineHandle;
 LogPointer : Pointer;
 LogProc : SVCengineLogProc) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
 '_CHMengineSetLogCallBackPerEngine@12';

function CHMengineSetLogCallback(Handle : TCHMengineHandle;
 LogProc : SVClogProc) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
 '_CHMengineSetLogCallBack@8';

// These methods provide version information.
function CHMengineGetMajorVersion(Handle : TCHMengineHandle; Version : TLongIntPtr)
 : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
 '_CHMengineGetMajorVersion@8';

function CHMengineGetMinorVersion(Handle : TCHMengineHandle; Version : TLongIntPtr)
 : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
 '_CHMengineGetMinorVersion@8';

function CHMengineGetBuildNumber(Handle : TCHMengineHandle; Build : TLongIntPtr)
 : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
 '_CHMengineGetBuildNumber@8';

function CHMengineSetRejectBadSegmentGrammarFlag(Handle : TCHMengineHandle; Flag : Boolean)
 : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
 '_CHMengineSetRejectBadSegmentGrammarFlag@8';

function CHMengineMessageName
(
   Handle : TCHMengineHandle;
   MessageIndex : Cardinal;
   pName        : TPAnsiCharPtr
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
 '_CHMengineMessageName@12';

function CHMengineCountOfConfig( Handle : TCHMengineHandle; pCountOfConfig : TIntegerPtr )
 : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
 '_CHMengineCountOfConfig@8';

function CHMengineConfigName( Handle : TCHMengineHandle; ConfigIndex : Integer; pName : TPAnsiCharPtr )
 : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
 '_CHMengineConfigName@12';

function CHMengineGetIncomingConfigIndex( Handle : TCHMengineHandle; pIndex : TIntegerPtr )
 : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
 '_CHMengineGetIncomingConfigIndex@8';

function CHMengineGetOutgoingConfigIndex( Handle : TCHMengineHandle; pIndex : TIntegerPtr )
 : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
 '_CHMengineGetOutgoingConfigIndex@8';

function CHMengineSetCurrentConfig( Handle : TCHMengineHandle; Index : Integer )
 : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
 '_CHMengineSetCurrentConfig@8';

function CHMengineGetCurrentConfig( Handle : TCHMengineHandle; pIndex : TIntegerPtr )
 : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name
 '_CHMengineGetCurrentConfig@8';

// Error interface
function CHMerrorRelease(Handle : TCHMresultHandle)
 : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name '_CHMerrorRelease@4';

function CHMerrorGetDescription(Handle      : TCHMresultHandle;
                                Description : TPAnsiCharPtr)
 : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name '_CHMerrorGetDescription@8';

function CHMerrorGetCode(Handle  : TCHMresultHandle;
                         Code    : TLongIntPtr)
 : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name '_CHMerrorGetCode@8';


{$ENDIF} // HIDE

{: Get a message in a printable format by appending a newline character '\n' after
every carriage return character '\r'.}
function CHMconvertLineFeeds(Original : AnsiString) : AnsiString;

type
   {: This class handles Chameleon exceptions. }
   TCHMexception = class(Exception)
   public
      {: Constructor.}
      constructor Create(ErrorHandle : TCHMresultHandle);
   end;

{$IFNDEF HIDE}
procedure Register;
{$ENDIF}

implementation

procedure Register;
begin
   RegisterComponents('Chameleon', [TCHMdynamicEngine]);
end;

procedure CHMcheckResult(ReturnVal : TCHMresultHandle);
begin
   if TCHMresultHandle(0) <> ReturnVal then
   begin
      // here is a great place to put a break point
      Raise TCHMexception.Create(ReturnVal);
   end;
end;

procedure LogCallBack(LogHandle : Pointer; Line: PAnsiChar); stdcall;
var
  Engine : TCHMcustomEngine;
begin
   if LogHandle <> nil then
   begin
      Engine := LogHandle;
      Engine.WriteToLog(Line);
   end;
end;

function CHMregistrationLevel
(
   pLevel : TLongIntPtr
) : TCHMresultHandle; stdcall; external 'CHM_LIB3.dll' name '_CHMregistrationLevel@4';

function ChameleonRegistrationLevel : LongInt;
begin
   CHMregistrationLevel(Addr(Result));
end;

constructor TCHMcustomEngine.Create(Owner: TComponent);
var
   StubEnginePointer : Pointer;
begin
   StubEnginePointer := self;
   FEngineHandle := nil;
   inherited Create(Owner);
   try
      CHMcheckResult(CHMengineCreate(Addr(EngineHandle)));
   except on E : Exception do
      begin
         ShowMessage(E.Message);
         raise E;
      end;
   end;
   CHMcheckResult(CHMengineAddRef(EngineHandle));
   CHMcheckResult(CHMengineSetRejectBadSegmentGrammarFlag(EngineHandle, FStrictGrammarChecking));
   CHMcheckResult(CHMengineSetLogCallbackPerEngine(EngineHandle, StubEnginePointer, LogCallBack) );
end;

destructor TCHMcustomEngine.Destroy;
begin
   if EngineHandle <> nil then
   begin
      CHMcheckResult(CHMengineSetLogCallbackPerEngine(EngineHandle, nil, LogCallBack));
      CHMcheckResult(CHMengineRelease(EngineHandle));
   end;
   inherited Destroy;
end;

procedure TCHMcustomEngine.HandleUnexpectedMessage(MessageData: AnsiString; TransportIndex, MessageIndex: Integer);
begin
   // Change this procedure in the parent class to alter the behavior
   if Assigned(FOnParseError) then
   begin
      FOnParseError(Self, MessageData, TransportIndex, 'Unexpected message encountered - this should not occur - contact iNTERFACEWARE support.', CHM_DEF_prUnknownMessage);
   end;
end;

procedure TCHMcustomEngine.WriteToLog(Line: AnsiString);
begin
   if Assigned(FOnLog) then
      TCHMlogEvent(FOnLog)(Self, Line);
end;

procedure TCHMcustomEngine.SetOnLog(const Value: TCHMlogEvent);
begin
  FOnLog := Value;
end;

procedure TCHMcustomEngine.SetRejectBadSegmentGrammarFlag(const Value: Boolean);
begin
   FStrictGrammarChecking := Value;
   CHMcheckResult(CHMengineSetRejectBadSegmentGrammarFlag(EngineHandle, FStrictGrammarChecking));
end;

{ TCHMexception }

constructor TCHMexception.Create(ErrorHandle: TCHMresultHandle);
var
   pDescription : PAnsiChar;
begin
   if ErrorHandle = TCHMresultHandle(-1) then
   begin
      inherited Create('Call failed.');
      HelpContext := -1;
   end
   else
   begin
      inherited Create('');
      CHMerrorGetCode(ErrorHandle, Addr(HelpContext));
      CHMerrorGetDescription(ErrorHandle, Addr(pDescription));
      Self.Message := pDescription;
      CHMerrorRelease(ErrorHandle);
   end;
end;

{ TCHMcustomTableRow }

function TCHMcustomTableRow.CountOfSubTable: Integer;
begin
   Result := FTable.CountOfSubTable;
end;

constructor TCHMcustomTableRow.Create(Table: TCHMcustomTable; RowIndex: Integer);
begin
   inherited Create;
   Assert(RowIndex < Table.CountOfRow);
   FTable := Table;
   FRowIndex := RowIndex;
   FDateTime := nil;
end;

destructor TCHMcustomTableRow.Destroy;
begin
   FDateTime.Free;
   inherited Destroy;
end;

function TCHMcustomTableRow.GetSubTable(SubTableIndex: Integer): TCHMcustomTable;
begin
   Assert(FTable <> nil);
   Assert(FTable.CountOfSubTable > SubTableIndex);
   Result := FTable.SubTable[FRowIndex, SubTableIndex];
end;

function TCHMcustomTableRow.RowIndex: Integer;
begin
   Result := FRowIndex;
end;

function TCHMcustomTableRow.TableHandle: TCHMtableHandle;
begin
   Result := FTable.TableHandle;
end;

{ TCHMcustomTable }

function TCHMcustomTable.AddRow : Integer;
begin
   Assert(TableHandle <> nil);
   CHMcheckResult(CHMtableAddRow(TableHandle));
   Result := CountOfRow-1;
end;

function TCHMcustomTable.ColumnName(ColumnIndex: Integer): AnsiString;
var
   pColumnName : PAnsiChar;
begin
   CHMcheckResult(CHMtableColumnName(FTableHandle, ColumnIndex, Addr(pColumnName)));
   Result := pColumnName;
end;

function TCHMcustomTable.ColumnType(ColumnIndex: Integer): LongInt;
begin
   CHMcheckResult(CHMtableColumnType(FTableHandle, ColumnIndex, Addr(Result)));
end;

function TCHMcustomTable.CountOfColumn: Integer;
begin
   CHMcheckResult(CHMtableCountOfColumn(FTableHandle, Addr(Result)));
end;

function TCHMcustomTable.CountOfRow: Integer;
begin
   CHMcheckResult(CHMtableCountOfRow(FTableHandle, Addr(Result)));
end;

function TCHMcustomTable.CountOfSubTable: Integer;
begin
   InitSubTables(0);
   Result := FCountOfSubTable;
end;

constructor TCHMcustomTable.Create(Handle: TCHMtableHandle);
begin
   inherited Create;
   FTableMatrixRowCount := 0;
   FTableRow := nil;
   FTableMatrix := nil;
   FTableHandle := Handle;
   if FTableHandle = nil then
   begin
      CHMcheckResult(CHMtableCreate(Addr(FTableHandle)));
   end;

   CHMcheckResult(CHMtableAddRef(FTableHandle));
end;

procedure TCHMcustomTable.DeleteSubTables;
var
   TableIndex, RowIndex : Integer;
begin
   if (FTableMatrix <> nil) then
   begin
      for TableIndex := 0 to FCountOfSubTable -1 do
      begin
         for RowIndex := 0 to FTableMatrixRowCount - 1 do
         begin
            TCHMcustomTable((Pointer(Integer(FTableMatrix) + sizeof(Pointer) * (RowIndex * FCountOfSubTable + TableIndex)))^).Free;
         end;
      end;
      FreeMem(FTableMatrix);
      FTableMatrix := nil;
   end;
end;

destructor TCHMcustomTable.Destroy;
begin
   DeleteSubTables;
   FTableRow.Free;
   CHMcheckResult(CHMtableRelease(FTableHandle));
   inherited Destroy;
end;

function TCHMcustomTable.Dump: AnsiString;
var
   pContent : PAnsiChar;
begin
   CHMcheckResult(CHMtableDump(TableHandle, Addr(pContent)));
   Result := pContent;
   CHMcheckResult(CHMtableFreeBuffer(Addr(pContent)));
end;

function TCHMcustomTable.GetDateTimeValue(RowIndex,
  ColumnIndex: Integer): TCHMdateTime;
var
   Handle : TCHMdateTimeHandle;
begin
   CHMcheckResult(CHMtableGetDateTime(TableHandle, ColumnIndex, RowIndex, Addr(Handle)));
   try
      FDateTime.Free;
      FDateTime := TCHMdateTime.Create(Handle);
   except
      on E: Exception do
      begin
         FDateTime := nil;
         Raise E;
      end;
   end;

   Result := FDateTime;
end;

function TCHMcustomTable.GetDoubleValue(RowIndex,
  ColumnIndex: Integer): double;
begin
   CHMcheckResult(CHMtableGetDouble(TableHandle, ColumnIndex, RowIndex, Addr(Result)));
end;

function TCHMcustomTable.GetIntegerValue(RowIndex,
  ColumnIndex: Integer): Integer;
begin
   CHMcheckResult(CHMtableGetInteger(TableHandle, ColumnIndex, RowIndex, Addr(Result)));
end;

function TCHMcustomTable.IsNull(RowIndex, ColumnIndex : Integer) : boolean;
begin
   CHMcheckResult(CHMtableIsNull(TableHandle, ColumnIndex, RowIndex, Addr(Result)));
end;

function TCHMcustomTable.GetState(RowIndex, ColumnIndex : Integer) : Integer;
begin
   CHMcheckResult(CHMtableState(TableHandle, ColumnIndex, RowIndex, Addr(Result)));
end;

function TCHMcustomTable.GetRow(RowIndex: Integer): TCHMcustomTableRow;
begin
   FTableRow.Free;
   FTableRow := TCHMcustomTableRow.Create(Self, RowIndex);
   Result := FTableRow;
end;

function TCHMcustomTable.GetStringValue(RowIndex,
  ColumnIndex: Integer): AnsiString;
var
   Value : PAnsiChar;
begin
   CHMcheckResult(CHMtableGetString(TableHandle, ColumnIndex, RowIndex, Addr(Value)));
   Result := Value;
end;

function TCHMcustomTable.GetSubTable(RowIndex,
  SubTableIndex: Integer): TCHMcustomTable;
begin
   InitSubTables(RowIndex);
   Assert(SubTableIndex < FCountOfSubTable);
   Assert(FTableMatrix <> nil);
   Assert(Pointer((Pointer(Integer(FTableMatrix) + RowIndex * FCountOfSubTable + SubTableIndex))^) <> nil);
   Result := TCHMcustomTable((Pointer(Integer(FTableMatrix) + sizeof(Pointer) * (RowIndex * FCountOfSubTable + SubTableIndex)))^);
end;

procedure TCHMcustomTable.InitSubTables(RowIndexRequired : Integer);
var
//   CountOfRow : Integer;
   TableIndex, RowIndex : Integer;
   SubTableHandle : TCHMtableHandle;
begin
   if FTableMatrix <> nil then
   begin
      if RowIndexRequired >= FTableMatrixRowCount then
      begin
         DeleteSubTables;
      end
      else
      begin
         exit;
      end;
   end;
   CHMcheckResult(CHMtableCountOfSubTable(FTableHandle, Addr(FCountOfSubTable)));
   CHMcheckResult(CHMtableCountOfRow(FTableHandle, Addr(FTableMatrixRowCount)));
   FTableMatrix := AllocMem(CountOfRow * FCountOfSubTable * sizeof(Pointer));
   for TableIndex :=0 to FCountOfSubTable -1 do
   begin
      for RowIndex := 0 to FTableMatrixRowCount -1 do
      begin
         CHMcheckResult(CHMtableSubTable(FTableHandle, RowIndex, TableIndex, Addr(SubTableHandle)));
         Pointer((Pointer(Integer(FTableMatrix) + sizeof(Pointer) * (RowIndex * FCountOfSubTable + TableIndex)))^) := TCHMcustomTable.Create(SubTableHandle);
      end;
   end;

end;

function TCHMcustomTable.TableHandle: TCHMtableHandle;
begin
   Result := FTableHandle;
end;

function CHMconvertLineFeeds(Original : AnsiString) : AnsiString;
var
   CharIndex : Integer;
   Output : string;
begin
   Output := '';
   for CharIndex := 1 to Length(Original) do
   begin
      Output := Output + Original[CharIndex];
      if Original[CharIndex] = AnsiChar(13) then
         Output := Output + Chr(10);
   end;
   Result := Output;
end;

function TCHMcustomTable.TableName: AnsiString;
var
   pName : PChar;
begin
   CHMcheckResult(CHMtableName(TableHandle, Addr(pName)));
   Result := pName;
end;

{ TCHMdateTime }

constructor TCHMdateTime.Create(Handle: TCHMdateTimeHandle);
begin
   inherited Create;
   FHandle := Handle;
   CHMcheckResult(CHMdateTimeAddRef(Handle));
end;

destructor TCHMdateTime.Destroy;
begin
   CHMcheckResult(CHMdateTimeRelease(FHandle));
   inherited Destroy;
end;

function TCHMdateTime.GetDateTime: TDateTime;
begin
   Assert(FHandle <> nil);
   CHMcheckResult(CHMdateTimeGetDATE(FHandle, Addr(Result)));
end;

function TCHMdateTime.GetSecondFraction: Double;
begin
   Assert(FHandle <> nil);
   CHMcheckResult(CHMdateTimeGetSecondFraction(FHandle, Addr(Result)));
end;

function TCHMdateTime.GetZone: SmallInt;
begin
   Assert(FHandle <> nil);
   CHMcheckResult(CHMdateTimeGetZone(FHandle, Addr(Result)));
end;

function TCHMdateTime.GetIsNull : Boolean;
var
   Value : ShortInt;
begin
   Assert(FHandle <> nil);
   CHMcheckResult(CHMdateTimeGetIsNull(FHandle, Addr(Value)));
   if Value = 1 then
   begin
      Result := true;
   end
   else
   begin
      Result := false;
   end;
end;

procedure TCHMdateTime.SetDateTime(const Value: TDateTime);
begin
   Assert(FHandle <> nil);
   CHMcheckResult(CHMdateTimeSetDATE(FHandle, Value));
end;

procedure TCHMdateTime.SetSecondFraction(const Value: Double);
begin
   Assert(FHandle <> nil);
   CHMcheckResult(CHMdateTimeSetSecondFraction(FHandle, Value));
end;

procedure TCHMdateTime.SetZone(const Value: SmallInt);
begin
   Assert(FHandle <> nil);
   CHMcheckResult(CHMdateTimeSetZone(FHandle, Value));
end;

{ TCHMcustomMessage }

function TCHMcustomMessage.AddRow : Integer;
begin
   Result := inherited AddRow;
end;

constructor TCHMcustomMessage.Create(iEngineHandle: TCHMengineHandle;
  iTableHandle: TCHMtableHandle; iMessageIndex: Integer);
begin
   inherited Create(iTableHandle);
   if iTableHandle = nil then
   begin
      CHMcheckResult(CHMengineCreateEmptyMessageTable(iEngineHandle, iMessageIndex, TableHandle));
      AddRow;
   end;
   FEngineHandle := iEngineHandle;
   FMessageIndex := iMessageIndex;
end;

constructor TCHMcustomMessage.CreateFromMessageIndex(Engine : TCHMcustomEngine;
  iMessageIndex: Integer);
begin
   Create(Engine.EngineHandle, nil, iMessageIndex);
end;

function TCHMcustomMessage.GenerateFlatWire: AnsiString;
var
   FlatWire : PAnsiChar;
begin
   Assert(FEngineHandle <> nil);
   CHMcheckResult(CHMengineGenerateMessage(FEngineHandle, Addr(FlatWire), FMessageIndex, FTableHandle));
   Result := FlatWire;
end;

procedure TCHMcustomMessage.SetEngineHandle(const Value: TCHMengineHandle);
begin
  FEngineHandle := Value;
end;

procedure TCHMcustomEngine.Load(FileName : AnsiString);
var
   AnsiFileName: AnsiString;
begin
   AnsiFileName := FileName;
   CHMcheckResult(CHMengineLoad(EngineHandle, PAnsiChar(AnsiFileName)));
//   CHMcheckResult(CHMengineLoad(EngineHandle, PChar(FileName)));
   FMappingFile := FileName;
end;

procedure TCHMcustomEngine.SetMappingFile(const Value : AnsiString);
begin
   if (Value <> '') then
   begin
      Load(Value);
   end;
end;


procedure TCHMcustomEngine.SetOnMessage(
  const Value: TCHMmessageEvent);
begin
  FOnMessage := Value;
end;

procedure TCHMcustomEngine.ParseMessage(MessageData: AnsiString;
  TransportIndex: Integer);
begin
   if Assigned(FOnMessage) then
      FOnMessage(Self, MessageData);
end;

function TCHMcustomEngine.TransformMessage(InMessage: AnsiString;
  InConfigIndex, OutConfigIndex: Cardinal): AnsiString;
var
  MessageIndex : Cardinal;
  OutMessage : PAnsiChar;
begin
   CHMcheckResult(CHMengineTransformMessage(EngineHandle,
                                            PAnsiChar(InMessage),
                                            InConfigIndex,
                                            OutConfigIndex,
                                            Addr(OutMessage),
                                            Addr(MessageIndex),
                                            0));
   Result := AnsiString(OutMessage);
end;

function TCHMcustomEngine.TranslateHL7toXML(InMessage : AnsiString) : AnsiString;
var
   pResult : PAnsiChar;
begin
   CHMcheckResult(CHMengineTranslateHL7toXML(EngineHandle,
                                             PAnsiChar(InMessage),
                                             Addr(pResult)));
   Result := pResult;
end;

function TCHMcustomEngine.TranslateXMLtoHL7(InXML : AnsiString) : AnsiString;
var
   pResult : PAnsiChar;
begin
   CHMcheckResult(CHMengineTranslateXMLtoHL7(EngineHandle,
                                             PAnsiChar(InXML),
                                             Addr(pResult)));
   Result := pResult;
end;

procedure TCHMcustomEngine.SetOnParseError(
  const Value: TCHMparseErrorEvent);
begin
  FOnParseError := Value;
end;

function TCHMcustomEngine.GetMajorVersion : LongInt;
begin
   CHMcheckResult(CHMengineGetMajorVersion(EngineHandle, Addr(Result)));
end;

function TCHMcustomEngine.GetMinorVersion : LongInt;
begin
     CHMcheckResult(CHMengineGetMinorVersion(EngineHandle, Addr(Result)));
end;

function TCHMcustomEngine.GetBuildNumber : LongInt;
begin
     CHMcheckResult(CHMengineGetBuildNumber(EngineHandle, Addr(Result)));
end;

function TCHMcustomEngine.GetCountOfConfig : Integer;
begin
   CHMcheckResult(CHMengineCountOfConfig(EngineHandle, Addr(Result)));
end;

function TCHMcustomEngine.GetConfigName( Index : Integer ) : AnsiString;
var
   Value : PAnsiChar;
begin
   CHMcheckResult(CHMengineConfigName(EngineHandle, Index, Addr(Value)));
   Result := Value;
end;

function TCHMcustomEngine.GetIncomingConfigIndex : Integer;
begin
   CHMcheckResult(CHMengineGetIncomingConfigIndex(EngineHandle, Addr(Result)));
end;

function TCHMcustomEngine.GetOutgoingConfigIndex : Integer;
begin
   CHMcheckResult(CHMengineGetOutgoingConfigIndex(EngineHandle, Addr(Result)));
end;

function TCHMcustomEngine.GetCurrentConfigIndex : Integer;
begin
   CHMcheckResult(CHMengineGetCurrentConfig(EngineHandle, Addr(Result)));
end;

procedure TCHMcustomEngine.SetCurrentConfigIndex( Index : Integer );
begin
   CHMcheckResult(CHMengineSetCurrentConfig(EngineHandle, Index));
end;

procedure TCHMcustomEngine.HandleException(
  MessageData: AnsiString;
  TransportIndex : Integer;
  ExceptionHandle: TCHMresultHandle);
var
   pDescription : PAnsiChar;
   Description : AnsiString;
   ErrorCode : Longint;
begin
   if ExceptionHandle = TCHMresultHandle(-1) then
   begin
      Description := 'Call failed!';
      ErrorCode := -1;
   end
   else
   begin
      CHMerrorGetCode(ExceptionHandle, Addr(ErrorCode));
      CHMerrorGetDescription(ExceptionHandle, Addr(pDescription));
      Description := pDescription;
      CHMerrorRelease(ExceptionHandle);
   end;
   if Assigned(FOnParseError) then
   begin
      FOnParseError(Self, MessageData, TransportIndex, Description, TErrorCode(ErrorCode));
   end
   else
   begin
      WriteToLog(Description);
   end;
end;

{ TCHMdynamicEngine }

procedure TCHMdynamicEngine.ParseMessage(MessageData: AnsiString;
  TransportIndex: Integer);
begin
   inherited ParseMessage(MessageData, TransportIndex);
   if Assigned(FOnParseMessage) then
   begin
      FOnParseMessage(Self, MessageData, TransportIndex);
   end;
end;

function TCHMdynamicEngine.ParseMessageDynamically(MessageData: AnsiString;
  MessageTable: TCHMcustomMessage): Integer;
begin
   CHMcheckResult(CHMengineParseMessage(EngineHandle, PAnsiChar(MessageData), Addr(Result), MessageTable.TableHandle));
end;

procedure TCHMdynamicEngine.SetOnParseMessage(
  Event: TCHMparseMessageEvent);
begin
   FOnParseMessage := Event;
end;

{ TCHMdynamicMessage }

constructor TCHMdynamicMessage.Create(Engine : TCHMcustomEngine);
begin
   inherited Create(Engine.FEngineHandle, nil, 0);
end;

constructor TCHMdynamicMessage.CreateFromMessageIndex(Engine : TCHMcustomEngine; MessageIndex : Integer);
begin
   inherited Create(Engine.FEngineHandle, nil, MessageIndex);
end;


function TCHMcustomEngine.MessageName(MessageIndex: Cardinal): AnsiString;
var
   pName : PAnsiChar;
begin
   CHMcheckResult(CHMengineMessageName(FEngineHandle, MessageIndex, Addr(pName)));
   Result := pName;
end;

function TCHMcustomEngine.AckMessageIndex: Cardinal;
begin
   CHMcheckResult(CHMengineGetAckMessageIndex(FEngineHandle, Addr(Result)));
end;

function TCHMcustomEngine.GenerateAck(OriginalMessage: AnsiString): AnsiString;
var
   pAck : PAnsiChar;
begin
   CHMcheckResult(CHMengineGenerateAckFromMessage(FEngineHandle, PAnsiChar(OriginalMessage), Addr(pAck)));
   Result := pAck;
end;

end.
